#!/usr/bin/perl

use strict vars;
use vars qw(
$GV
$GVportal
$PRM
@modes
$DBH
$zeroPos
$GENEMARK_speciesModel
$GENSCAN_speciesModel
%EV
);

########################################
### main
########################################
$PRM->{USERid} = &{$GV->{getUserIdFunction}};


########################################
### Source Data Functions
########################################


# init function

sub init_das{
    require 'das_scripts/dasFunctions.pl';
    dasParam(); # function to set chr, start and stop from cookie
    return;
}


# get evidence functions

sub getEvidence_db{
    # this function queries the local evidence database for the following information and the results are returned as an array
    # [database name],[method of generation],[evidence name],[unique id],[genome start position],[genome end position],[score],[exon number],[evidence reference],[strand (1,0)],[color]
    # The evidence reference is a url to the primary data of the evidence, such as an alignment, program output, 
    #
    # This example has two queries for two tables that contain different types of evidence
    # The query results are added to $evidenceHashRef using the getExons function
    # this evidenceHashRef is returned
    my %evidenceHash;
    my $evidenceHashRef = \%evidenceHash;

   
    for my $r ( @{&{$GV->{evidenceSources}}} ){
	my $ref2=$GV->{DBH}->selectall_arrayref($r->[2]);
	$evidenceHashRef = getExons($ref2,$r->[0],$r->[1],$evidenceHashRef);
    }

  return $evidenceHashRef;
}

sub getEvidence_das{
    my $evidenceHasRef = &queryDASevidence_genomeSequence();
    return $evidenceHasRef;
}

# get genome sequence functions

sub getGenomeSequence_das{
    return $PRM->{GenomeSequence}; # already defined and set in getEvidence_das
}

sub getGenomeSequence_ex{
    # this example uses a fasta formatted file as genome sequence and takes a simple perl substring
    # larger implementations should replace this with fastacmd, or an alternative function
    my $seq = `more ../db/zm.fsa`;
    $seq =~ s/^.+?\n.+?\n.+?\n.+?\n//s;
    $seq =~ s/\n//sg;
    $seq = substr($seq,$PRM->{start}-1,($PRM->{end} - $PRM->{start}) +1);
    return $seq;
}

sub getImageMap{
    require 'makeImage.pl';
    my $imagelinkALL = &returnImageMap();
    $imagelinkALL .= "<script>var GenomeSequence = '$PRM->{GenomeSequence}';</script>";
    return $imagelinkALL;
}

# get scale

sub getScale{
my $zeroPos = int($PRM->{start});
my $StartX=10;
my $Margin=10;

my $imgWidth = $PRM->{imgWidth};

my $scale = ($PRM->{end} - $zeroPos + 1) / ($PRM->{imgWidth} - 2*$Margin);

#my $scale = $seqLen / $rulerLen;

$scale = sprintf("%.4f", $scale);

return ($scale,$zeroPos,$StartX,$Margin); # [base to pixel scale] , [base position of start of graphic], [left padding white space in graphic

}

########################################
### End Source Data Functions
########################################

########################################
### Database
########################################

# source database
my %attr = (PrintError=>1,RaiseError=>1);

sub getUserId{
  my @sessionCookie = cookie($GV->{SessCookieName}); # reads session id from cookie and accesses php session variables
  my $id = $sessionCookie[0];
  if ($id ne ""){
    my $session = PHP::Session->new($id, { create => 1,save_path => $GV->{session_path} });
    if ($session->is_registered($GV->{SessLoginParam})){
      my $USERid = $session->get($GV->{SessLoginParam});
      return $USERid;
    }else{
      $session->destroy;
      return;
    }
  }
  return;
}

sub logout{
  my @sessionCookie = cookie($GV->{SessCookieName}); # reads session id from cookie and accesses php session variables
  my $id = $sessionCookie[0];
  my $cookie;
  if ($id ne ""){
    my $session = PHP::Session->new($id, { create => 1, save_path => $GV->{session_path} });
    $session->destroy;
    $cookie = cookie(-name=>$GV->{SessCookieName},-value=>$session->id,-expires=>'-1d');
  }
  return $cookie;
}

sub login{
  # simple user verification (not secure)
  my ($username,$passwd) = @_;
  if ($username eq "" && $passwd eq ""){
      return 0;
  }
  my $sql = "select user_name from users where user_name = '$username' and pword = password('$passwd') ";
  my @arr = $GV->{LDBH}->selectrow_array($sql);
  if ($arr[0] eq $username && $arr[0] ne ""){
    srand;
    my $session = PHP::Session->new(int(rand()*10000000000),{ create => 1,save_path => $GV->{session_path},auto_save => 1 });
    $session->set($GV->{SessLoginParam} => $username);
    my $scookie = cookie(-name=>$GV->{SessCookieName},-value=>$session->id); # deleted at end of session
    return $scookie;
  }
  return 0;  
}

sub user_register{
    my ($username,$passwd,$fullname,$phone,$email) = @_;
    my $account_type = 'USER';
    my $sql = "insert into users (user_name,pword,fullname,phone,email) VALUES ('$username',password('$passwd'),'$fullname','$phone','$email')";
    my $sth = $GV->{ADBH}->do($sql) || return 0; # returns false if insert failed; unique keys in users table determine valid record
    return 1;
}


sub getAdminOwnership{
  # not checked
  # find annotations currently checked out by administrator, returns hash{geneid} = admin who checked it out

  my $sql = "select uid, max(checked_out_date) as maxDate from admin_session where dbName='$GV->{dbTitle}' group by uid";
  my $latestREF = $GV->{LDBH}->selectall_hashref($sql,'uid');

  my $expire_secs = 60*30;

  my %ownedHash = ();

  for my $k (keys %$latestREF){
    my $sql = "select uid,USERid,geneName from admin_session where uid = '$k' and checked_out_date = '$$latestREF{$k}{'maxDate'}' and (UNIX_TIMESTAMP(now()) - UNIX_TIMESTAMP(checked_out_date))  < $expire_secs and dbName = '$GV->{dbTitle}' and returned = 0";
    my $oREF = $GV->{LDBH}->selectall_hashref($sql,'uid');
    if (keys(%$oREF)){  
      $ownedHash{$k} = $$oREF{$k}{'USERid'};
    }
  }
  return \%ownedHash;
}


sub getUserGroup{
    # distinguish admin and other groups
    my $sql = "select account_type from users where user_name = '$PRM->{USERid}'";
    my @ref = $GV->{LDBH}->selectrow_array($sql);
    if ($ref[0] eq "ADMIN"){
	return 1;
    }else{
	return 0;
    }
}

sub addUserAnnotation{
  my ($stat) = @_;
  my $info = $PRM->{info};
  my $strand = ($info =~ /^comp/)? 'r': 'f';
  my ($UAstart) = $info =~ /^[^\d]+(\d+)/;
  my ($UAend) = $info =~ /(\d+)\)+$/;
  $PRM->{mRNAseq} =~ s/\s//g; # no whitespace for sequence entries
  $PRM->{proteinseq} =~ s/\s//g; 

  my $sql = "insert into user_gene_annotation (USERid,geneId,chr,strand,l_pos,r_pos,gene_structure,description,CDSstart,CDSstop,proteinId,geneAliases,proteinAliases,status,modDate,evidence,annotation_type,mRNAseq,proteinseq,GSeqEdits,organism,dbName,dasCookie,dbVer)";
  $sql .= " VALUES (\"$PRM->{USERid}\",\"$PRM->{UCAannid}\",\"$PRM->{chr}\",'$strand',$UAstart,$UAend,\"$PRM->{info}\",\"$PRM->{desc}\",'$PRM->{cds_start}','$PRM->{cds_end}',\"$PRM->{prod}\",\"$PRM->{geneAlias}\",\"$PRM->{protAlias}\",'$stat',NOW(),\"$PRM->{Esource}\",\"$PRM->{annotation_type}\",\"$PRM->{mRNAseq}\",\"$PRM->{proteinseq}\",\"$PRM->{GSeqEdits}\",\"$GV->{speciesName}\",\"$GV->{dbTitle}\",\"".cookie("DASsel")."\",\"$PRM->{dbVer}\");";

  my $sth = $GV->{ADBH}->do($sql) || return 0; # returns 1 if successful update, 0 if unsuccessful, # geneId is unique index in database
  $sql = "select last_insert_id();";
  my ($id) = $GV->{ADBH}->selectrow_array($sql);
  $PRM->{uid} = $id;
  if ($stat eq "SUBMITTED_FOR_REVIEW"){
    AdminNotify();
  }
  return 1;
}


sub removeAnnotation{
 my ($USERid, $uid) = @_;
 
 if (($PRM->{owner} eq $PRM->{USERid})||( getUserGroup($PRM->{USERid}) )){ # if owned or isAdmin
 my $sql = "delete from user_gene_annotation where uid = '$PRM->{uid}'";
 my $sth = $GV->{ADBH}->do($sql) || bailOut("mysql error: $sql");
 return 1;
 }
 bailOut("Delete action not permitted");
}

sub updateAnnotation{
  my ($stat) = @_;
  my $strand = ($PRM->{info} =~ /^comp/)? 'r': 'f';
  my $info = $PRM->{info};
  my ($UAstart) = $info =~ /^[^\d]+(\d+)/;
  my ($UAend) = $info =~ /(\d+)\)+$/;
  $PRM->{mRNAseq} =~ s/\s//g; # no whitespace for sequence entries
  $PRM->{proteinseq} =~ s/\s//g;
  my $sql = "update user_gene_annotation set USERid = \"".(($PRM->{owner})?$PRM->{owner}:$PRM->{USERid})."\" ,geneId = '$PRM->{UCAannid}', chr =\"$PRM->{chr}\", strand = \"$strand\",l_pos = $UAstart, r_pos = $UAend, gene_structure = \"$PRM->{info}\", description = \"$PRM->{desc}\", CDSstart = \"$PRM->{cds_start}\", CDSstop = \"$PRM->{cds_end}\", proteinId = \"$PRM->{prod}\", geneAliases = \"$PRM->{geneAlias}\", proteinAliases = \"$PRM->{protAlias}\", status = \"$stat\", annotation_type = \"$PRM->{annotation_type}\", mRNAseq = \"$PRM->{mRNAseq}\", proteinseq =\"$PRM->{proteinseq}\", modDate = NOW(), GSeqEdits = \"$PRM->{GSeqEdits}\",evidence = \"$PRM->{Esource}\", dasCookie=\"$PRM->{dasCookie}\", dbVer=\"$PRM->{dbVer}\"  where uid = $PRM->{uid}";
  my $sth = $GV->{ADBH}->do($sql) || return 0;
  if ($stat eq "SUBMITTED_FOR_REVIEW"){
    AdminNotify();
  }

  return 1; # returns 1 if successful update, 0 if unsuccessful, # geneId is unique index in database
}

sub loadUCA {
# loads UCA values from database

  my $ucaQuery=qq{
		  select uid,USERid,geneId,proteinId,chr,l_pos,r_pos,gene_structure,description,geneAliases,proteinAliases,CDSstart,CDSstop,status,DATE_FORMAT(modDate,'\%W, \%M \%d, \%Y'),DATE_FORMAT(modDate,'\%r'),evidence,mRNAseq,proteinseq,GSeqEdits,organism,dasCookie,dbVer
		  from user_gene_annotation
		  where uid = \"$PRM->{uid}\"
		 };
  my @arr = $GV->{ADBH}->selectrow_array($ucaQuery);
  if ( $#arr < 0 ){
      return 0;
  }

  my ($uid,$USERid,$geneId,$proteinId,$chr,$l_pos,$r_pos,$gene_structure,$desc,$geneAliases,$proteinAliases,$cds_start,$cds_end,$status,$modDate,$modTime,$Esource,$mRNA,$protein,$GSeqEdits,$organism,$dasCookie,$dbVer) = @arr;
  $PRM->{owner} = $USERid;
  $PRM->{uid} = $uid;
  $PRM->{UCAannid} = $geneId;
  $PRM->{chr} = $chr;
  $PRM->{info} = $gene_structure;
  $PRM->{desc} = $desc;
  $PRM->{Esource} = $Esource;
  $PRM->{geneAlias} = $geneAliases;
  $PRM->{protAlias} = $proteinAliases;
  $PRM->{prod} = $proteinId;
  $PRM->{modifyState} = $status;
  $PRM->{modDate} = $modDate;
  $PRM->{modTime} = $modTime;
  $PRM->{status} = $status;
  ($PRM->{start},$PRM->{end}) = $gene_structure =~ /\((\d+).+?(\d+)\)/;
  $PRM->{cds_start} = $cds_start;
  $PRM->{cds_end} = $cds_end;
  $PRM->{mRNAseq} = $mRNA;
  $PRM->{proteinseq} = $protein;
  $PRM->{GSeqEdits} = $GSeqEdits;
  $PRM->{organism} = $organism;
  $PRM->{dasCookie} = $dasCookie;  # only for das input
  $PRM->{dbVer} = $dbVer;
  return 1;
}

########################################
### end Database
########################################

####################################################################
# General Functions
####################################################################


sub extraAdminSubmit{
    if (!$GV->{email}){
	return;
    }

    # sends email to Annotator and Administrator
    my $AdminEmail = $GV->{AdminEmail};
    my $emailTXT = $PRM->{emailTXT};
    
    
    my $sql = "select fullname,email from users where user_name = '$PRM->{owner}' ";
    my $ref = $GV->{LDBH}->selectall_arrayref($sql);
    my $fullname = $ref->[0]->[0];
    my $userEmail = $ref->[0]->[1];   
    if ( ($userEmail =~ /(.+?)\@(.+)/) && ($AdminEmail =~ /(.+?)\@(.+)/ ) ){ # email address check, send will crash if email address is not in correct format
    open(MAIL, "| /usr/sbin/sendmail -t");
 
    my $mailTXT .= "To: $userEmail\n";
    $mailTXT .= "CC: $AdminEmail\n";
    $mailTXT .= "From: $AdminEmail\n";
    $mailTXT .= "Subject: Accepted User Contributed Annotation at $GV->{dbTitle}\n\n";
    $mailTXT .= "Dear $fullname, \n\n";
    $mailTXT .= "Your annotation $PRM->{UCAannid} has been ";
    $mailTXT .= ($PRM->{status} eq "ACCEPTED") ? "accepted.\n\n" : "rejected.  \n\n";
    $mailTXT .= "Administrator comments:\n-------------------------------\n $emailTXT \n-------------------------------\n";
#    $mailTXT .= "Gene Annotation: <a href=''>$PRM->{UCAannid}</a>\n";
    $mailTXT .= "Region: ".(&{$GV->{GenomeContextLinkFunction}}($PRM->{chr},$PRM->{start},$PRM->{end},$PRM->{dbVer}))."\n";
    $mailTXT .= "Thank you for submitting a gene annotation to the $GV->{speciesName} at $GV->{dbTitle} community.\n\n\n".$GV->{HTMLpath};    

    print MAIL $mailTXT;
    close(MAIL);

    } # end if valid email address
    return;

}


sub AdminNotify{
    if (!$GV->{email}){
	return;
    }
    # sends email to Annotator and Administrator
    my $AdminEmail = $GV->{AdminEmail};
    my $emailTXT = $PRM->{emailTXT};

    
    if ( ($AdminEmail =~ /(.+?)\@(.+)/ ) ){ # email address check, send will crash if email address is not in correct format
    open(MAIL, "| /usr/sbin/sendmail -t");
 
    my $mailTXT .= "To: $AdminEmail\n";
    $mailTXT .= "From: $AdminEmail\n";
    $mailTXT .= "Subject: New User Contributed Annotation at $GV->{dbTitle} to be Reviewed\n\n";
    $mailTXT .= "Administrators,\n Please review this new User Contributed Annotation, $PRM->{UCAannid}, at $GV->{dbTitle}, submitted by $PRM->{USERid}.\n";
    $mailTXT .= "Region: ".(&{$GV->{GenomeContextLinkFunction}}($PRM->{chr},$PRM->{start},$PRM->{end},$PRM->{dbVer}))."\n";
    $mailTXT .= "\n\nAdministration Checkout (login required):\n http://www.plantgdb.org$GV->{CGIPATH}AdminAnnotation.pl";

    print MAIL $mailTXT;
    close(MAIL);

    } # end if valid email address
    return;

}


sub printEditLink{
  # add init function to establish global log in parameters?
  my $link = ($PRM->{USERid} eq $PRM->{owner} || $PRM->{status} ne "ACCEPTED") ? "<a href='$GV->{CGIPATH}AnnotationTool.pl?uid=$PRM->{uid}$GV->{tmpLinkParam}' target=_blank>edit</a>" : "";
}

sub printAnnotation{
  my $info = $PRM->{info};
  my $strand = ($info =~ /^comp/)? 'r': 'f';
  my ($UAstart) = $info =~ /^[^\d]+(\d+)/;
  my ($UAend) = $info =~ /(\d+)\)+$/;
  my $modifyDate = ($PRM->{modDate}) ? $PRM->{modDate} : localtime(time);
  my $pcr = ($PRM->{cds_start}) ? "$PRM->{cds_start}-$PRM->{cds_end}" : "";
  my $org = $PRM->{organism} || $GV->{speciesName};
  my $ps = $PRM->{proteinseq}; # removes line breaks when called from AnnotationTool.pl
  $ps =~ s/\W//g;
  my $ns = $PRM->{mRNAseq};
  $ns =~ s/\W//g;

  my $txt =<<END_TXT;
Gene Name\t$PRM->{UCAannid}
Annotator\t$PRM->{USERid}
Modify Date\t$modifyDate
Organism\t$org
Chromosome/Genome Segment\t$PRM->{chr}
Strand\t$strand
Left Position\t$UAstart
Right Position\t$UAend
Gene Structure\t$PRM->{info}
Protein Coding Region\t$pcr
mRNA sequence\t$ns
Protein Sequence\t$ps
Description\t$PRM->{desc}
Exon Origins\t$PRM->{Esource}
Putative Protein Product\t$PRM->{prod}
Gene Aliases\t$PRM->{geneAliases}
Protein Aliases\t$PRM->{proteinAliases}
Genome Sequence Edits\t$PRM->{GSeqEdits}

END_TXT

    return $txt;
}

sub printDetail{
  my $info = $PRM->{info};
  my $strand = ($info =~ /^comp/)? 'r': 'f';
  my ($UAstart) = $info =~ /^[^\d]+(\d+)/;
  my ($UAend) = $info =~ /(\d+)\)+$/;
  my $modifyDate = ($PRM->{modDate}) ? "$PRM->{modDate} $PRM->{modTime}": strftime( "%A, %B %d, %Y",localtime(time));
  my $pcr = ($PRM->{cds_start} > 0) ? "$PRM->{cds_start}-$PRM->{cds_end}" : "";
  my $esource_links = $PRM->{Esource};
  $esource_links =~ s/<newline>/\n/g;
#  $esource_links =~ s/(http:.+?)([<\s])/<a href="$1">$1<\/a>$2/g;
 $esource_links =~ s/(http:\S+)/<a href="$1">$1<\/a> /g;
  my $org = $PRM->{organism} || $GV->{speciesName};

  my $pf = strToFASTA($PRM->{proteinseq});
  my $nf = strToFASTA($PRM->{mRNAseq});
  my $txt =<<END_TXT;
<b>Gene Name</b>
$PRM->{UCAannid}

<b>Annotator:</b>
$PRM->{owner}

<b>Last Modified:</b>
$modifyDate

<b>Organism</b>
$org

<b>Chromosome / Genome Segment:</b>
$PRM->{chr}

<b>Strand:</b>
$strand

<b>Left Position:</b>
$UAstart

<b>Right Position:</b>
$UAend

<b>Gene Structure:</b>
<div style="height:50px;width:500px;overflow:auto">
$PRM->{info}
</div>

<b>Protein Coding Region:</b>
$pcr

<b>mRNA sequence:</b>
<div style="height:100px;width:500px;overflow:auto;font-family:Courier;">
$nf
</div>

<b>Protein Sequence:</b>
<div style="height:100px;width:500px;overflow:auto;font-family:Courier;">
$pf
</div>

<b>Description:</b>
$PRM->{desc}

<b>Exon Origins:</b>
[start][stop][source of exon][score][database name][parent unique Id][parent name][link to parent]
<div style="height:100px;overflow:auto; border:thin black solid;text-align:left">
$esource_links
</div>
<b>Putative Protein Product:</b>
$PRM->{prod}

<b>Gene Aliases:</b>
$PRM->{geneAliases}

<b>Protein Aliases:</b>
$PRM->{proteinAliases}

<b>Genome Sequence Edits:</b>
$PRM->{GSeqEdits}


END_TXT

return $txt;
}

sub seqToFASTA{
    my ($seqType) = @_;
    my $str .= ($seqType eq "p") ? strToFASTA($PRM->{proteinseq}) : strToFASTA($PRM->{mRNAseq});
    if ($str ne ""){
      $str = ">yrGATE|$PRM->{uid}|$PRM->{UCAannid}\n$str";
    }
    return $str;
}

sub strToFASTA{
    my ($seq) = @_;
    my $str;
    my $linelength = 60;
    for (my $i=0;$i<length($seq)/$linelength;$i++){
	$str .= substr($seq,$i*$linelength,$linelength)."\n";
    }
    #$str .= substr($seq,int(length($seq)/$linelength)*$linelength,length($seq))."\n";
    return $str;

}


sub yrgateToGFF3{
#GFF3 format defined at http://song.sourceforge.net/gff3.shtml
#[seqid] [source] [type] [start] [end] [score] [strand] [phase] [attributes]
#attributes = id,name,alias,parent,target,note,Dbxref,Evidence_for,Organism,Substituted_sequence,Inserted_sequence
#   Evidence_for,Organism is novel attribute for yrGATE GFF export
#types = gene,mRNA,exon,CDS,region,insertion,deletion,substitution

my ($local_uidRef) = @_;

my $fields_legal_regex = "[a-zA-Z0-9. :^*$@!+_?-]";
my $seqid_legal_regex = "[a-zA-Z0-9.:^*$@!+_?-|]";

my %local_uid = %$local_uidRef;
my @gff;
my $strand = ($PRM->{strand} eq "f") ? "+" : "-";

my $seq_id = $PRM->{chr};

# sequence edits
if ($PRM->{GSeqEdits}){
    # add note to GFF file
    $gff[++$#gff] = ["\#note: the genome segment for the following gene annotation has been edited"];
    $gff[++$#gff] = ["\#note: see http://www.plantgdb.org/yrGATE/genome_edits.txt for further description"];


    my $region_id = "region".(($local_uid{'region'}) ? $local_uid{'region'}++ : 1);
    my $region_attrib = "ID=$region_id;Name=$region_id;Organism=$PRM->{organism};Submitted_by=$PRM->{USERid}";
    # start is always 1 of edited regions
    $gff[++$#gff] = [escape_fields($PRM->{chr},'yrGATE','region','1',$PRM->{end},'.',$strand,'.',$region_attrib)];
    $seq_id = $region_id;
    my @edits = split /\n/, $PRM->{GSeqEdits};
    for my $i (@edits){
	    #chop $i; # ^M character
	    my @f = split /\,/, $i;
	    $f[2] =~ s/\W//; # any trailing characters
	    my $edit_type = ($f[1] eq "change") ? "substitution" : ($f[1] eq "insert") ? "insertion" : "deletion";
	    # start = end,  except insertions greater than length 1
  	    my $edit_length = (length($f[2]) > 1 and $edit_type eq "substitution") ? length($f[2]) : 0;
	    my $edit_id = "sequence_edit".(($local_uid{'edit'}) ? $local_uid{'edit'}++ : 1);
	    my $edit_attrib = "ID=$edit_id;Name=$edit_id;Parent=$region_id;";
	    if ($edit_type eq "insertion"){
		$edit_attrib .= "Inserted_sequence=$f[2];";
	    }elsif($edit_type eq "substitution"){
		$edit_attrib .= "Substituted_sequence=$f[2];";
	    }
	    $gff[++$#gff] = [escape_fields($region_id,'yrGATE',$edit_type,$f[0],($f[0]+$edit_length),'.',$strand,'.',$edit_attrib)];
    }
}



my $gene_id = "gene".( ($local_uid{'gene'}++ == 0) ? 1 : $local_uid{'gene'});
my $gene_attrib = "ID=$gene_id;Name=$PRM->{UCAannid};Organism=$PRM->{organism};Submitted_by=$PRM->{owner}";
$gff[++$#gff] = [escape_fields($seq_id,'yrGATE','gene',$PRM->{start},$PRM->{end},'.',$strand,'.',$gene_attrib)];
my %exon_idHash;
# add sequence edits


# for exons
my @exons = $PRM->{info} =~ /\d+\.\.\d+/g;

my @CDSgff;
my $extra = 0; # first CDS has phase 0
for my $i (@exons){
    my ($start,$end) = split /\.\./, $i;
    my $exon_id = "exon".( ($local_uid{'exon'}++ == 0) ? 1 : $local_uid{'exon'});
    $exon_idHash{"$start..$end"} = $exon_id;
    my $exon_attrib = "ID=$exon_id;Parent=$gene_id";
    $gff[++$#gff] = [escape_fields($seq_id,'yrGATE','exon',$start,$end,'.',$strand,'.',$exon_attrib)];
    # if CDS overlap
    if (min($PRM->{cds_start},$PRM->{cds_end}) <= $end && max($PRM->{cds_start},$PRM->{cds_end}) >= $start){
        my $cds_id = "CDS".( ( $local_uid{'cds'}++ == 0 ) ? 1 : $local_uid{'cds'} );
	my $cds_attrib = "ID=$cds_id;Parent=$gene_id;Name=$PRM->{prod};";
	my $cds_start = (min($PRM->{cds_start},$PRM->{cds_end}) <= $start) ? $start : min($PRM->{cds_start},$PRM->{cds_end});
	my $cds_end = ( max($PRM->{cds_start},$PRM->{cds_end}) <= $end) ? max($PRM->{cds_start},$PRM->{cds_end}) :  $end;

	$CDSgff[++$#CDSgff] = [escape_fields($seq_id,'yrGATE','CDS',$cds_start,$cds_end,'.',$strand,$extra,$cds_attrib)];

        $extra =  3 - ( $cds_end - $cds_start + 1 - $extra) % 3;  # bases remaining for last codon

        if ($extra == 3){
	    $extra = 0;
	}
    }
}
push @gff,@CDSgff;

if (1){
# add exon_origins
my @evidence = split /<newline>/, $PRM->{Esource};
for my $i (@evidence){
    my ($start,$end,$method,$score,$dbName,$id1,$id2,$ref_url) = split /\s/, $i;
    my $ev_id = "evidence".(($local_uid{'ev'}++ == 0) ? $local_uid{'ev'}++: $local_uid{'ev'});
    my $ev_attrib = "ID=$ev_id;Evidence_for=".$exon_idHash{min($start,$end)."..".max($start,$end)}.";";
    #add Target, DBxref attributes
    $gff[++$#gff] = [escape_fields($seq_id,$dbName,$method,min($start,$end),max($start,$end),$score,$strand,'.',$ev_attrib)];
}
}

my $gff3;
for my $j (@gff){
    $gff3 .= join("\t",@$j)."\n";
}
return ($gff3,\%local_uid);
}

sub escape_fields{
    # escapes fields for gff3
    my @fields = @_;
    my @escaped_fields = @fields;
    return @fields;
}


my ($CHR_SELECT_BOX,$RangeChrFIELD);

sub getExons{
# adds array of evidence to eH and EV
# eH organizes data keyed by exon
# EV organizes data keyed by evidence type
#
# returns reference to eH

my ($ref,$seqType,$color,$evidenceHashRef) = @_; #[reference to array of evidence], [current evidenceHash reference], [type of evidence]
my %eH = %$evidenceHashRef;
for (my $i=0;$i<scalar(@{$ref});$i++){
  my ($dbName,$exon_method,$seqName,$seqId,$exon_start,$exon_stop,$exon_score,$exon_num,$exon_link,$seqStrand) = @{$ref->[$i]};

  #if ((min($exon_start,$exon_stop) >= $zeroPos)&&(max($exon_start,$exon_stop) <= $PRM->{end} )){ # exons within window
  if (1){
  my $eKey = "$exon_start  $exon_stop"; # two spaces for exon key

  if ($eH{$eKey}){
    $eH{$eKey}->{maxScore} = ($exon_score > $eH{$eKey}->{maxScore}) ? $exon_score :  $eH{$eKey}->{maxScore};
    $eH{$eKey}->{members}->{"$seqId"} = {
                                           name   => $seqName,
			                   score  => $exon_score,
    			                   number => $exon_num,  # not used so far
				           method => $exon_method,
					   link   => $exon_link,
					   dbName => $dbName  
                                        };
  }else{
      $eH{$eKey} = {
	            maxScore => $exon_score,
 	            members  => {
 	                             "$seqId"=>{
			                           name   => $seqName,
			                           score  => $exon_score,
    			                           number => $exon_num,
				                   method  => $exon_method,
                                                   link   => $exon_link,
						   dbName => $dbName,
						   containedInRange => ((min($exon_start,$exon_stop) >= $zeroPos)&&(max($exon_start,$exon_stop) <= $PRM->{end} ))
		                                 }
		                 },
		    containedInRange => ((min($exon_start,$exon_stop) >= $PRM->{start})&&(max($exon_start,$exon_stop) <= $PRM->{end} ))
                 };

  }

## for drawing ambiguous strand from das data    $EV{$seqType}->{$seqId}->{strand} = ($seqStrand eq "+") ? 1 : ($seqStrand eq "-") ? -1 : 0;
    $EV{$seqType}->{$seqId}->{strand} = ($seqStrand eq "+") ? 1 : ($seqStrand eq "-") ? -1 : 0; # for image drawing
    $EV{$seqType}->{$seqId}->{exons}->{"${exon_start}..${exon_stop}"} = $exon_score;
    $EV{$seqType}->{$seqId}->{link} = $exon_link;
    $EV{$seqType}->{$seqId}->{db} = $GV->{dbTitle};
    $EV{$seqType}->{color} = $color;



  } # end if within window
} # end for

return \%eH;
}

sub getEvidenceTable{
my ($evidenceHashref) = @_;
my %evidenceHash = %$evidenceHashref;
my $groupcount = 1;
my @evidencelist = sort by_coord keys %evidenceHash;

my @cellWidths = ('10px','120px','45px','210px');

my $etableJscript = "";
my $eTable = "<table style='border:gray solid thin;font-family:Arial;font-size:10px;padding:0px;spacing:0px;empty-cells:hide' width='420px'>
<tr><td colspan=4><font class=s2>Evidence Table<\/font>&nbsp&nbsp&nbsp <input class=utButton type=button onclick='clickHideExons();' value='only display selected exons' id='hideButton'><\/td><\/tr>
<tr><td width='$cellWidths[0]'><\/td><td align=center width='$cellWidths[1]'>Exon Coordinates</td><td width='$cellWidths[2]'>Score<\/td><td width='$cellWidths[3]' align=center>Evidence supporting exon<\/td><\/tr>
<tr><td colspan=4>
<div id='eTable' style='position=relative;top:0;left:0;width:420px;height:$GV->{eTableHeight}px;overflow:auto'>
<table width='400px' !class=mainTable style='spacing:0px;padding:0px;border:0px;margin:0px;background:lightskyblue'>";
my $memberseqs = "";
my %evidenceHashC = %evidenceHash;

foreach my $k (@evidencelist){ # each group of evidence table
  my ($groupMin, $groupMax);
  if ($evidenceHash{$k}){
      if ($evidenceHash{$k}->{containedInRange}==0){
	  next;  # only include those exons fully contained in range
      }
    my @coord = split /  /, $k;
    my $rowcolor = ($groupcount % 2) ? "#ededb1":"#CCCC99";
    $groupMin = min($coord[0],$coord[1]);
    $groupMax = max($coord[0],$coord[1]);

    $eTable .= "\n\n\n<tr style='padding:0px;spacing:0px;' id=\"frow".min($coord[0],$coord[1])."  ".max($coord[0],$coord[1])."\"><td valign=middle style='width:$cellWidths[0];font-size:10px;'>$groupcount</td><td>
<table id=\"table".min($coord[0],$coord[1])."  ".max($coord[0],$coord[1])."\" style=\"background:lavender;padding:0px;spacing:0px;text-align;left;font-family:Arial;font-size:10px;border:thin gray solid;width:".($cellWidths[1] + $cellWidths[2] + $cellWidths[3])."\"><tr id=\"".min($coord[0],$coord[1])."  ".max($coord[0],$coord[1])."\"><td width='$cellWidths[1]'><input type=radio name='e$groupcount' value='".min($coord[0],$coord[1])."  ".max($coord[0],$coord[1])."' onClick='selectradio($groupcount,\"".min($coord[0],$coord[1])."  ".max($coord[0],$coord[1])."\",1);'";
    $eTable .= ($PRM->{info} =~ /$coord[1]\.\.$coord[0]/ || $PRM->{info} =~ /$coord[0]\.\.$coord[1]/) ? " checked=checked " : "";
    $eTable .= ">".min($coord[0],$coord[1])."  ".max($coord[0],$coord[1])."<\/td><td width='$cellWidths[2]'>".$evidenceHash{$k}->{maxScore}.'</td><td width="$cellWidths[3]">'.&evidence_links($evidenceHash{$k}->{members})."<\/td><\/tr>";

    #$etableJscript .= "eTableExons['".min($coord[0],$coord[1])."  ".max($coord[0],$coord[1])."'] = '".&TypeList(\%evidenceHash,$k)."';\n";
    $etableJscript .= &TypeList(\%evidenceHash,$k);
    my $InmembersHR = $evidenceHash{$k}->{members};
    delete $evidenceHash{$k};
    delete $evidenceHashC{$k};
    my ($InmembersHR2) ; # tmp variable should be deleted!

    ($groupMin,$groupMax,$InmembersHR2) = GroupExon(min($coord[0],$coord[1]),max($coord[0],$coord[1]),$InmembersHR2,\%evidenceHashC); # gets min and max coordinates for exon group

    my @evidencelist2 = sort by_coord keys %evidenceHash;
    foreach my $k2 (@evidencelist2){
     if ($evidenceHash{$k2}){
      if ($evidenceHash{$k2}->{containedInRange}==0){
	  next;  # only include those exons fully contained in range
      }
      my @coord2 = split /  /, $k2;
      if ( ($groupMin <= max($coord2[0],$coord2[1])) and ($groupMax >= min($coord2[0],$coord2[1])) and ( SeqShare($InmembersHR,$evidenceHash{$k2}->{members}) )  ){
 
        $eTable .= "<tr id=\"".min($coord2[0],$coord2[1])."  ".max($coord2[0],$coord2[1])."\"><td width='$cellWidths[1]'><input type=radio name='e$groupcount' value='".min($coord2[0],$coord2[1])."  ".max($coord2[0],$coord2[1])."' onClick='selectradio($groupcount,\"".min($coord2[0],$coord2[1])."  ".max($coord2[0],$coord2[1])."\",1);'";
        $eTable .= ">".min($coord2[0],$coord2[1])."  ".max($coord2[0],$coord2[1])."<\/td><td width='$cellWidths[2]'>".$evidenceHash{$k2}->{maxScore}.'</td><td width="$cellWidths[3]">'.&evidence_links($evidenceHash{$k2}->{members})."<\/td><\/tr>";

        #test line for above # $eTable .= ">"."keys ".keys(%$InmembersHR)." $InmembersHR ".join(",",keys(%{$InmembersHR}))."<br> ".min($coord2[0],$coord2[1])."  ".max($coord2[0],$coord2[1])."<\/td><td>".$evidenceHash{$k2}->{maxScore}.'</td><td>'.&evidence_links($evidenceHash{$k2}->{members})."<\/td><\/tr>";
        AddSeqs($InmembersHR,$evidenceHash{$k2}->{members});
        #$etableJscript .= "eTableExons['".min($coord2[0],$coord2[1])."  ".max($coord2[0],$coord2[1])."'] = '".&TypeList(\%evidenceHash,$k2)."';\n";
        $etableJscript .= &TypeList(\%evidenceHash,$k2);
        delete $evidenceHash{$k2};
        delete $evidenceHashC{$k2};
      }
     }
    }
    $eTable .= "<\/table><\/td><\/tr>";
    $groupMin = $groupMax = "";
    $groupcount++;
  }
}

$eTable .= '</table></div></td></tr></table>';
$eTable .= "<script>var groupAmt = $groupcount;<\/script>";

return ($eTable,$etableJscript);
}


sub GroupExon{
    # tiles out boundaries of exon group, returns members of group and boundaries
    my ($groupMin, $groupMax, $membersHR, $eHashRefT) = @_;  #,$evidenceHashRef) = @_;
    my %eHashT = %{$eHashRefT};
    for my $k (keys %eHashT){
      my @coord = split /  /, $k;

      if ( ($groupMin <= max($coord[0],$coord[1])) and ($groupMax >= min($coord[0],$coord[1])) and &SeqShare($membersHR,$eHashT{$k}->{members})   ){
        AddSeqs($membersHR,$eHashT{$k}->{members});
        $groupMin = min($groupMin,min($coord[0],$coord[1]));
        $groupMax = max($groupMax,max($coord[0],$coord[1]));
	delete $eHashT{$k};
      }
    }
    if ( keys(%eHashT) == keys(%$eHashRefT) ){  # return if range gets all overlapping exons
      return ($groupMin,$groupMax,$membersHR);
    }else{ # recurse to expand range
      ($groupMin,$groupMax,$membersHR) = GroupExon($groupMin,$groupMax,$membersHR,\%eHashT);
    }
}


sub evidence_links{
    # links for evidence, in evidence table
    my ($membersHR) = @_;
    my $link_str;
    for my $k (keys %$membersHR){
	$link_str .= "<a href='$membersHR->{$k}->{link}' target='_blank'>$k</a>&nbsp\n";
    }
    return $link_str;
}


sub SeqShare{
  # checks for member duplicity within an exon variant group, else returns list of merged members
  my ($membersHR,$newmembersHR) = @_;
  for my $k (keys %$newmembersHR){
      if ($membersHR->{$k}){
          my %tmp = ();
	  return 0;
      }
  }
  return 1;
}

sub AddSeqs{
  my ($membersHR,$newmembersHR) = @_;
  for my $k (keys %$newmembersHR){
      $membersHR->{$k} = $newmembersHR->{$k};
  }
  return;
}

sub TypeList(){
  # format of Esource; returns Esource line for an exon
  my ($evidenceHR,$exon) = @_;
  my @coord = split /  /,$exon;
  my $linkstr;
  my $membersHR = $evidenceHR->{$exon}->{members};
  for my $k (keys %{$membersHR}){
    $linkstr .= ( ($linkstr ne "") ? "<newline>" : "").min($coord[0],$coord[1])." ".max($coord[0],$coord[1])." $membersHR->{$k}->{method} $membersHR->{$k}->{score} $membersHR->{$k}->{dbName} $k $membersHR->{$k}->{name} $membersHR->{$k}->{link}";
  }
  $linkstr = "eTableExons['".min($coord[0],$coord[1])."  ".max($coord[0],$coord[1])."'] = \"$linkstr\";\n";
  return $linkstr;
}


sub recordTable{
    my ($UCAref,$statusArrRef) = @_;
    my %statusHash;
    my $ownedRef = getAdminOwnership();
    my $linkTarget = " target=\"_blank\" ";
    my $page .= "<table class=mainT border=1><tr class='headRow'><td>Annotation ID</td><td>Organism</td><td>Context</td><td>Genome Segment</td><td>View Annotation Record</td><td>Download Annotation</td><td>Last Modified</td><td>Annotator</td></tr>";
    my $tablePage = "";
    for (my $i=0;$i<scalar(@$UCAref);$i++){
	my ($uid,$geneId,$modDate,$status,$owner,$chr,$l_pos,$r_pos,$proteinSeq,$organism,$dbName,$dbVer) = @{$UCAref->[$i]}; 
	if ($statusHash{$status}==0){
	    $tablePage .= "<tr class='catRow'><td colpsan=5>$status</td></tr>\n";
	    $statusHash{$status} = 1;
	}

  if ( !$$ownedRef{$uid} || ($$ownedRef{$uid} eq $PRM->{USERid})){ # check to see if under review
    $tablePage .= "<tr><td><a  target=\"_blank\" onClick=\"checkout('$uid');\">$geneId </a>&nbsp&nbsp";
    $tablePage .=  ($status ne "ACCEPTED") ? "<a $linkTarget href='$GV->{CGIPATH}AnnotationTool.pl?uid=$uid'>edit</a>" : "";
    $tablePage .= "</td>";
  }else{
    $tablePage .= "<tr class='checked'><td>$geneId</td>";
  }

  $tablePage .= "<td>$organism</td>";
  $tablePage .= "<td><a href='".(&{$GV->{GenomeContextLinkFunction}}($chr,$l_pos,$r_pos,$dbVer))."' $linkTarget>$dbName</a></td>\n";
  $tablePage .= "<td>$chr:$l_pos..$r_pos</td>";
  $tablePage .= "<td><a $linkTarget href='$GV->{CGIPATH}AnnotationDetail.pl?uid=$uid'>text</a></td>";
  $tablePage .= "<td><a $linkTarget href='$GV->{CGIPATH}AnnotationExport.pl?uid=$uid&html=1&format=fasta&html=1&seqType=n'>Nucleotide FASTA</a><br>";
  $tablePage .= ($proteinSeq ne "") ? "<a $linkTarget href='$GV->{CGIPATH}AnnotationExport.pl?uid=$uid&html=1&format=fasta&html=1&seqType=p'>Protein FASTA</a><br>\n" : "";
  $tablePage .= "<a $linkTarget href='$GV->{CGIPATH}AnnotationExport.pl?uid=$uid&html=1&format=gff3&html=1'> GFF3</a></td>\n";
  $tablePage .= "<td>$modDate</td><td>$owner</td>";

  $tablePage .= "</tr>\n";
}

# for empty status
for my $k (@$statusArrRef){
    if ($statusHash{$k} == 0){
      $tablePage .= "<tr><td class='catRow' colpsan=5>$k</td></tr>\n";
      $statusHash{$k} = 1;	
    }
}

$tablePage .= "</table><br>";
$page .= $tablePage;

return $page;
}

sub bailOut{
    my ($txt) = @_;
  $txt = ($txt eq "") ? "Improper parameters. Please enter the Gene Annotation Tool from a legitimate link." : $txt;

    print header();
    print "<html><head><LINK type='text/css' rel='stylesheet' href='$GV->{HTMLPATH}yrGATE.css'></head><body>";
    print printTitle("error",0,1);
    print $txt;
    print "</body></html>";
    exit();
}

sub min{
  my ($a,$b) = @_;
  if ($a > $b){
    return $b;
  }else{
    return $a;
  }
}

sub max{
  my ($a,$b) = @_;
  if ($a < $b){
    return $b;
  }else{
    return $a;
  }
}

sub by_coord{
  my @s1 = split /  /,$a;
  my @s2 = split /  /,$b;
  $s1[0] <=> $s2[0]
    ||
  $s1[1] <=> $s2[1]
}

sub printTitle{
  my ($title,$lines,$nav) = @_;
  my $titleText;
  if ($GV->{TitleHeader}){
     $titleText .= $GV->{TitleHeader};
  }
  $titleText .= "<div class=headtitle><font style='font-weight:bold;font-size:24px'>yrGATE: $title </font><br>";
  if ($lines){
  $titleText .= "<font style='font-size:14px;padding:0px;'>&nbsp;&nbsp;&nbsp;<b>Data</b>: $GV->{dbTitle} &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
  $titleText .= ($GV->{speciesName}) ? "<b>Organism</b>: $GV->{speciesName}<br>" : "<br>";
  $titleText .= ($PRM->{USERid}) ? "&nbsp;&nbsp;&nbsp;<b>Annotator</b>: $PRM->{USERid}<br>" : "";
  }
  $titleText .= "</font><br>";

  if ($nav){
      my $space = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
      $titleText .= "<nobr>$space<font class=navlink>";
      $titleText .= "<a href='$GV->{CGIPATH}CommunityCentral.pl'>Community Annotation Central</a>$space";
      $titleText .= ($GV->{login_required}) ? (($PRM->{USERid}) ? "<a href='$GV->{CGIPATH}AnnotationAccount.pl'>Annotation Account</a>$space"."<a href='$GV->{CGIPATH}logout.pl'>Logout</a>" : "<a style='text-decoration:none;' href='$GV->{CGIPATH}login.pl'>Login</a>".$space."<a style='text-decoration:none;' href='$GV->{CGIPATH}userRegister.pl'>Sign up for an Account</a>" ): "";
      $titleText .= (&{$GV->{getUserGroupFunction}}($PRM->{USERid})) ? "$space<a href='$GV->{CGIPATH}AdminAnnotation.pl'>Administrate Annotations</a>": "";
      $titleText .= "$space<a href='mailto:$GV->{AdminEmail}'>Help</a>";
      $titleText .= "</font></nobr>";
  }

  $titleText .= "</div><br><br>";

  return $titleText;
}


sub printFooter{
    my $text;
    $text = "<br><br>";
    $text .= "yrGATE (c) 2006";
    $text .= "<br><a href='$GV->{CGIPATH}browser_support.pl'>supported browsers</a>";
    return $text;
}

sub disconnectDB{
    if ($GV->{LDBH}) {$GV->{LDBH}->disconnect;}
    if ($GV->{ADBH}) {$GV->{ADBH}->disconnect;}
    if ($GV->{DBH}) {$GV->{DBH}->disconnect;}
}





return 1;
